"""
MSCI website Scrape

This script allows the user to scrape the companies' ESG ratings from the MSCI
website. Website link:
"https://www.msci.com/our-solutions/esg-investing/esg-ratings/esg-ratings-corporate-search-tool/"

This tool accepts Company's names list in comma separated value
file (.csv) format as input.

This script requires that `pandas` be installed within the Python
environment you are running this script in.

The output is a .csv file with Company name and its corresponding ESG ratings
"""

import pandas as pd
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.keys import Keys
from time import sleep
from tqdm import tqdm
from .scraper import WebScraper


def _append_dict(temp: str) -> str:
    ''' Append the MSCI dictionary with Company Name and its MSCI ESG rating

    Parameters
    ----------
    temp : str
    The previous company name appended to the dictionary

    Returns
    -------
    str
            The latest company name appended to the dictionary
    '''
    if temp == company:
        bot.append_empty_values(msci)

    else:
        msci['MSCI_Company'].append(company.text)
        msci['MSCI_ESG'].append(esg_score.get_attribute('class'))
        temp = company
    return temp


# Read input companies dataset
companies_filename = WebScraper._get_filename()
header_name = WebScraper._get_headername()
export_path = WebScraper._get_exportpath()
df = pd.read_csv(companies_filename)
data_length = len(df)

# Set up the webdriver
URL = "https://www.msci.com/our-solutions/esg-investing/esg-ratings/esg\
        -ratings-corporate-search-tool/"
bot = WebScraper(URL)

# Accept cookies on the website
cookies_xpath = '//*[@id="portlet_mscicookiebar_WAR_mscicookiebar"]/div/div[2]/ \
                div/div/div[1]/div/button[1]'
bot.accept_cookies(cookies_xpath)

# Extract company names and their ESG score and store it in the dictionary
temp = 0
for i in tqdm(range(data_length)):
    msci = {'MSCI_Company': [], 'MSCI_ESG': []}
    # Starting the search by finding the search bar & searching for the company
    search_bar = bot.send_request_to_search_bar(
        header_name, df, i, xpath='//*[@id="_esgratingsprofile_keywords"]')
    search_bar.send_keys(Keys.DOWN, Keys.RETURN)
    sleep(4)

    try:
        xpath = '//*[@id="_esgratingsprofile_esg-ratings-profile-header"]/div[2]/div[1]/div[2]/div'
        esg_score = bot.find_element(xpath)
        company = bot.find_element(
            '//*[@id="_esgratingsprofile_esg-ratings-profile-header"]/div[1]/div[1]')
        temp = _append_dict(temp)

    except NoSuchElementException:
        bot.append_empty_values(msci)

    # Save the data into a csv file
    df1 = bot.convert_dict_to_csv(msci, export_path)



'''
This module contains a class for uploading the data to a SQL relational
database.
'''
import pandas as pd
from sqlalchemy import create_engine


class RdsUploader:
    '''
    This class is used to upload data to a SQL relational database. An engine
    is created using the input information : database_type: str, dbapi: str,
    endpoint: str, port: int, database: str, user: str, password: str

    Attributes:
        database_type (str): It is an attribute that specifies the database
         type of any object
        dbapi (str): It's the library that lets Python connect to the database
         server
        endpoint (str): It's the computation resource that lets you run SQL
         commands on data objects within the Databricks environment
        port (int) : The port on which the database accepts connections
        database (str) : The database name
        user (str): The username that you configured for your database
        password (str): The password that you configured for your database

    '''

    # , database_type: str, dbapi: str, endpoint: str, port:
    def __init__(self):
        # int, database: str, user: str, password: str
        '''
        See help(RdsUploader) for accurate signature
        '''

        database_type = input('database_type : ')
        dbapi = input('dbapi : ')
        endpoint = input('endpoint : ')
        port = input('port : ')
        database = input('database : ')
        user = input('Username : ')
        password = input('Password : ')

        self.database_type = database_type
        self.dbapi = dbapi
        self.endpoint = endpoint
        self.port = port
        self.database = database
        self.user = user
        self.password = password
        self.engine = create_engine(f"{database_type}+{dbapi}://{user}\
                                    :{password}@{endpoint}:{port}/{database}")

    def send_query(self):
        '''
        This function does the engine execution
        '''
        self.engine.execute()

    def connection_check(self):
        '''
        This function checks the engine connection
        '''
        self.engine.connect()

    def create_table(self, df: pd.DataFrame, table_name: str):
        '''
        This function creates a SQL table from the input pandas
        dataframe

        Args:
            df (pd.DataFrame): Data in Pandas DataFrame format
            table_name (str): Name to be given to the SQL table
        '''
        df.to_sql(table_name, self.engine, if_exists='replace')

    def read_table(self, table_name: str) -> pd.DataFrame:
        '''
        This function reads the SQL table from the database and outputs Pandas
         DataFrame

        Args:
            table_name : Name of the SQL table

        Returns:
            pd.DataFrame: Pandas Dataframe generated from the SQL table
        '''
        df = pd.read_sql_table(table_name, self.engine)
        return df

    @staticmethod
    def add_rows(df: pd.DataFrame, df2: pd.DataFrame) -> pd.DataFrame:
        '''
        This function add rows from the Pandas DataFrame to the
        input Pandas DataFrame

        Args:
            df (pd.DataFrame): Input data in which rows to be added
            df2 (pd.DataFrame): Data whose rows to be added in df

        Returns:
            pd.DataFrame: Pandas Dataframe
        '''
        df = df.append(df2, ignore_index=True)
        return df

    @staticmethod
    def delete_row(df, labels) -> pd.DataFrame:
        '''
        This function deletes row from the Pandas DataFrame database

        Args:
            df (pd.DataFrame): Input data in which rows to be added
            labels (list) : list of the labels of rows to be deleted

        Returns:
            pd.DataFrame: Pandas Dataframe
        '''
        df1 = df.drop(labels=labels, axis=0, inplace=False)
        return df1



""" CSR HUB website Scrape

This script allows the user to scrape the companies' CSR ratings from
the CSR HUB website. Website link: "https://www.csrhub.com/search/name/"

This tool accepts Company's names list in comma separated value
file (.csv) format as input.

This script requires that `pandas` be installed within the Python
environment you are running this script in.

The output is a .csv file with Company name and its corresponding CSR ratings
"""

import pandas as pd
from time import sleep
from selenium.webdriver.common.keys import Keys
from selenium.common.exceptions import NoSuchElementException
from tqdm import tqdm
from .scraper import WebScraper


def _append_dict() -> dict:
    ''' Append the CSR dictionary with Company Name and its CSR score
    Returns
    -------
    dict
        The CSR dictionary
    '''
    try:
        csr_score = bot.find_element(
            '//*[@id="wrapper"]/div[3]/section[3]/div[2]/table/tbody/tr[2]/td[2]')
        csr['CSR_Ratings'].append(csr_score.text)
        company = bot.find_element(
            '//*[@id="wrapper"]/div[3]/section[3]/div[2]/table/tbody/tr[2]/td[1]/a')
        csr['CSR_Company'].append(company.text)

    except NoSuchElementException:
        bot.append_empty_values(csr)
    return csr


# Read input companies dataset

companies_filename = WebScraper._get_filename()
header_name = WebScraper._get_headername()
export_path = WebScraper._get_exportpath()
df = pd.read_csv(companies_filename)
data_length = len(df)


# Set up driver
URL = "https://www.csrhub.com/search/name/"
bot = WebScraper(URL)

# Accept cookies
cookies_xpath = '//*[@id="body-content-holder"]/div[2]/div/span[2]/button'
bot.accept_cookies(cookies_xpath)

# Scrape the website. Extract company names and their respective CSR score
i = 0
progress_bar = tqdm(total=data_length)
while i < data_length:
    csr = {'CSR_Company': [], 'CSR_Ratings': []}
    delay = 2  # seconds

    try:
        search_bar = bot.send_request_to_search_bar(
            header_name, df, i, xpath='//*[@id="search_company_names_0"]')
        search_bar.send_keys(Keys.RETURN)
        sleep(1)
        csr = _append_dict()
        # Save the data into a csv file
        df1 = bot.convert_dict_to_csv(csr, export_path)
        i += 1
    # If no element found, the page is restarted
    except NoSuchElementException:
        bot = bot.restart_driver(cookies_xpath)

    sleep(0.1)
    progress_bar.update(1)

progress_bar.close()



'''
The subpackage esgscraper contains modules:
'__main__', 'scraper', 'csrhub', 'snp_global', 'msci', 'sustainanalytics',\
'yahoo'
'''
__all__ = ['scraper', 'csrhub', 'snp_global', 'msci',
           'sustainanalytics', 'yahoo']



"""

This script allows the user to scrape the ESG, CSR ratings and financial
metrics of the companies from the following websites: SustainAnalytics,
S&P Global, CSR HUB, MSCI, Yahoo Finance

This tool accepts Company's names list in comma separated value
file (.csv) format as input.

The output is a .csv file with Company name and its corresponding ESG/
CSR ratings/Financial Metrics

To run on example dataset (Forbes.csv provided with this package), enter 0
for Companies filename and Header Name

This script will ask the user to input a number that pertains to the website
from which the information has to be scraped
"""
from .scraper import WebScraper

WebScraper.get_esgdata()



"""Yahoo Finance website Scrape

This script allows the user to scrape the companies' ESG ratings from
the Yahoo Finance website.
Website link: "https://finance.yahoo.com/lookup"

This tool accepts Company's names list in comma separated value
file (.csv) format as input.

This script requires that `pandas` be installed within the Python
environment you are running this script in.

The output is a .csv file with Company name and its corresponding
finance statistics and ESG ratings
"""

import pandas as pd
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.common.keys import Keys
from time import sleep
from tqdm import tqdm
from .scraper import WebScraper


def append_finance_data(temp: str, company_name: str) -> \
        str:
    ''' Append the Yahoo Finance dictionary with each company's\
        Financial statistics

    Parameters
    ----------
    temp : str
    The previous company name appended to the dictionary

    company_name : str
    The company name to be searched

    Returns
    -------
    str
        The latest company name appended to the dictionary
    '''
    if temp == company_name:
        bot.append_empty_values(fin)

    else:
        xpath_stats_button = '//*[@data-test="STATISTICS"]'
        bot.wait_element_to_load(xpath_stats_button)
        statistics_button = bot.find_element(xpath_stats_button)
        sleep(1)
        statistics_button.click()
        xpath_stats = '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/ \
                      div[2]/div[3]/div/div[2]/div/div/table/tbody/tr[2]/td[2]'
        bot.wait_element_to_load(xpath_stats)

        quar = bot.find_element(
            '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/div[3]/ \
            div/div[1]/div/div/table/tbody/tr[2]/td[2]')
        pm = bot.find_element(
            '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/div[3]/ \
             div/div[2]/div/div/table/tbody/tr[1]/td[2]')
        op = bot.find_element(
            '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/div[3]/ \
            div/div[2]/div/div/table/tbody/tr[2]/td[2]')
        roa = bot.find_element(
            '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/div[3]/ \
             div/div[3]/div/div/table/tbody/tr[1]/td[2]')
        roe = bot.find_element(
            '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/div[3]/ \
             div/div[3]/div/div/table/tbody/tr[2]/td[2]')
        eps = bot.find_element(
            '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/div[3]/ \
             div/div[4]/div/div/table/tbody/tr[7]/td[2]')
        cf = bot.find_element(
            '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/div[3]/ \
             div/div[6]/div/div/table/tbody/tr[1]/td[2]')
        d2e = bot.find_element(
            '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/div[3]/ \
             div/div[5]/div/div/table/tbody/tr[4]/td[2]')
        por = bot.find_element(
            '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/div[2]/ \
             div/div[3]/div/div/table/tbody/tr[6]/td[2]')
        st_p = bot.find_element(
            '//*[@id="quote-header-info"]/div[3]/div[1]/div/span[1]')

        fin["fin_Company"].append(company_name)
        xpath_mcap = '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/ \
                      div[1]/div/div/div/div/table/tbody/tr[1]/td[2]'
        mcap = bot.find_element(xpath_mcap)
        bot.wait_element_to_load(xpath_mcap)
        fin["Market Cap"].append(mcap.text)
        xpath_pe = '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/ \
                    div[1]/div/div/div/div/table/tbody/tr[3]/td[2]'
        pe = bot.find_element(xpath_pe)
        bot.wait_element_to_load(xpath_pe)
        fin["Trailing P/E"].append(pe.text)
        pb = bot.find_element(
            '//*[@id="Col1-0-KeyStatistics-Proxy"]/section/div[2]/div[1]/ \
            div/div/div/div/table/tbody/tr[7]/td[2]')
        fin["Price/Book (mrq)"].append(pb.text)
        fin["Most Recent Quarter (mrq)"].append(quar.text)
        fin["Profit Margin"].append(pm.text)
        fin["Op_Margin"].append(op.text)
        fin["ROA"].append(roa.text)
        fin["ROE"].append(roe.text)
        fin["Diluted EPS"].append(eps.text)
        fin["Operating_CashFlow"].append(cf.text)
        fin["Total Debt/Equity (mrq)"].append(d2e.text)
        fin["PayoutRat"].append(por.text)
        fin["Stock Price"].append(st_p.text)
        temp = company_name
    return temp


def append_esg_data():
    '''Append the Yahoo Finance dictionary with each company's ESG rating'''
    try:
        xpath_button_esg = '//*[@id="quote-nav"]/ul/li[11]/a/span'
        bot.wait_element_to_load(xpath_button_esg)
        button = bot.find_element(xpath_button_esg)
        button.click()
        xpath_esg = '//div[@class="Fz(36px) Fw(600) D(ib) Mend(5px)"]'
        bot.wait_element_to_load(xpath_esg)
        esg = bot.find_element(xpath_esg)
        fin["ESG"] = esg.text

    except NoSuchElementException:
        fin["ESG"] = None


def append_dict(temp: str):
    '''
    Append the Yahoo Finance dictionary with each company's
    Financial statistics and ESG rating

    Parameters
    ----------
    temp : str
    The previous company name appended to the dictionary

    Returns
    -------
    str
        The latest company name appended to the dictionary
    '''
    try:
        xpath = '//*[@id="quote-header-info"]/div[2]/div[1]/div[1]/h1'
        company_xpath = bot.find_element(xpath)
        bot.wait_element_to_load(xpath)
        company_name = company_xpath.text
        temp = append_finance_data(temp, company_name)
        append_esg_data()
        return temp

    except NoSuchElementException:
        bot.append_empty_values(fin)


# Read input companies dataset
companies_filename = WebScraper._get_filename()
header_name = WebScraper._get_headername()
export_path = WebScraper._get_exportpath()
df = pd.read_csv(companies_filename)
data_length = len(df)

# Set up driver
URL = "https://finance.yahoo.com/lookup"
bot = WebScraper(URL)

# Accept cookies
cookies_xpath = '//*[@id="consent-page"]/div/div/div/form/div[2]/div[2]/button'
bot.accept_cookies(cookies_xpath)

# Scrape the website. Extract company names and their respective CSR score
temp = 0
i = 0
progress_bar = tqdm(total=data_length)
while i < data_length:
    if i > 0 and i % 100 == 0:
        bot = bot.restart_driver(cookies_xpath)
    fin = {'fin_Company': [], 'Market Cap': [], 'Trailing P/E': [],
        'Price/Book (mrq)': [], 'Most Recent Quarter (mrq)': [],
        'Profit Margin': [], 'Op_Margin': [], 'ROA': [], 'ROE': [],
        'Diluted EPS': [], 'Operating_CashFlow': [],
        'Total Debt/Equity (mrq)': [], 'PayoutRat': [], 'Stock Price': [],
        'ESG': []}
    Company = df.loc[i][header_name]
    class_name = ' Bdrs(0) Bxsh(n)! Fz(s) Bxz(bb) D(ib) Bg(n) Pend(5px) Px(8px) Py(0) H(30px) Lh(30px) Bd O(n):f O(n):h Bdc($seperatorColor) Bdc($linkColor):f Bdc($c-fuji-punch-a):inv C($negativeColor):in M(0) Pstart(10px) Bxz(bb) Bgc(white) W(100%) H(32px)! Lh(32px)! Ff($yahooSansFinanceFont)'
    try:
        # Finding the search bar and searching for the company
        xpath = f'//*[@class="{class_name}"]'
        sleep(3)
        bot.wait_element_to_load(xpath)
        search_bar = bot.find_element(xpath)
        bot.wait_element_to_load(xpath)
        search_bar.clear()
        search_bar = bot.find_element(xpath)
        bot.wait_element_to_load(xpath)
        search_bar.send_keys(Company)
        sleep(5)
        search_bar = bot.find_element(xpath)
        bot.wait_element_to_load(xpath)
        search_bar.send_keys(Keys.DOWN)
        search_bar = bot.find_element(xpath)
        search_bar.send_keys(Keys.RETURN)
        sleep(5)
        temp = append_dict(temp)
        df1 = bot.convert_dict_to_csv(fin, export_path)
        i += 1

    except NoSuchElementException:
        # If the webpage changes, the driver is restarted
        bot = bot.restart_driver(cookies_xpath)

    sleep(1)
    progress_bar.update(1)

progress_bar.close()



""" S&P Global website Scrape

This script allows the user to scrape the companies' ESG ratings from the S&P
Global website
Website link: "https://www.spglobal.com/esg/scores/"

This tool accepts Company's names list in comma separated value
file (.csv) format as input.

This script requires that `pandas` be installed within the Python
environment you are running this script in.

The output is a .csv file with Company name and its corresponding ESG ratings
"""

import pandas as pd
from selenium.webdriver.common.keys import Keys
from selenium.common.exceptions import NoSuchElementException
from time import sleep
from tqdm import tqdm
from .scraper import WebScraper


def append_dict(temp: str) -> str:
    ''' Append the dictionary with Company name, Industry, Country, Ticker\
    and ESG rating

    Parameters
    ----------
    temp : str
    The previous company name appended to the dictionary

    Returns
    -------
    str
        The latest company name appended to the dictionary
    '''
    if temp == ESG_Company:
        bot.append_empty_values(SnP)

    else:
        ESG_Score = bot.find_element(xpath)
        SnP["SnP_ESG_Score"].append(ESG_Score.text)
        SnP["SnP_ESG_Company"].append(ESG_Company.text)
        ESG_Country = bot.find_element('//*[@id="company-country"]')
        SnP["SnP_ESG_Country"].append(ESG_Country.text)
        ESG_Industry = bot.find_element('//*[@id="company-industry"]')
        SnP["SnP_ESG_Industry"].append(ESG_Industry.text)
        ESG_Ticker = bot.find_element('//*[@id="company-ticker"]')
        SnP["SnP_ESG_Ticker"].append(ESG_Ticker.text)
        temp = ESG_Company
        return temp


# Read input companies dataset
companies_filename = WebScraper._get_filename()
header_name = WebScraper._get_headername()
export_path = WebScraper._get_exportpath()
df = pd.read_csv(companies_filename)
data_length = len(df)

# Set up driver
URL = "https://www.spglobal.com/esg/scores/"
bot = WebScraper(URL)

# Accept cookies
cookies_xpath = '//*[@id="onetrust-accept-btn-handler"]'
bot.accept_cookies(cookies_xpath)

# Scrape the website. Extract company names and their respective ESG score
temp = 0
for i in tqdm(range(data_length)):
    SnP = {'SnP_ESG_Company': [], 'SnP_ESG_Score': [],
           'SnP_ESG_Country': [], 'SnP_ESG_Industry': [], 'SnP_ESG_Ticker': []}

    try:
        # Starting the search by finding the search bar and searching for the
        #  company
        search_bar = bot.send_request_to_search_bar(
            header_name, df, i, class_name='banner-search__input')
        search_bar.send_keys(Keys.RETURN)
        sleep(4)
        xpath = '//*[@id="esg-score"]'
        bot.wait_element_to_load(xpath)
        ESG_Company = bot.find_element('//*[@id="company-name"]')
        temp = append_dict(temp)

    except NoSuchElementException:
        bot.append_empty_values(SnP)

    df1 = bot.convert_dict_to_csv(SnP, export_path)



from setuptools import setup
from setuptools import find_packages
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
setup(
    name="ESGScraper",
    version='1.0.0',
    description='Package that allows you to find ESG ratings from Yahoo\
    Finance, MSCI, CSR Hub, S&P Global, SustainAnalytics. In addition,\
    financial information is also scraped from Yahoo Finance',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/shweta-29/Companies_ESG_Scraper.git',
    download_url='https://github.com/shweta-29/Companies_ESG_Scraper/archive/refs/tags/v_01.tar.gz',
    author='Shweta Yadav',
    author_email='shweta.yadav2092@gmail.com',
    license='MIT',
    packages=find_packages(),
    install_requires=['selenium', 'pandas', 'tqdm', 'sqlalchemy'],
    include_package_data=True,
    package_data={'': ['data/*.csv']}
)



'''This module contains a class for scraping the websites.'''

import pandas as pd
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from selenium.common.exceptions import TimeoutException
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import StaleElementReferenceException
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.remote.webelement import WebElement
from time import sleep
import os
import pkg_resources


class WebScraper():

    '''
    This class is used to scrape a website.

    Attributes:
        URL (str): The website URL.
    '''

    def __init__(self, URL: str):
        '''
        See help(scraper) for accurate signature
        '''
        self.URL = URL
        options = Options()
        options.add_argument("--headless")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-dev-shm-usage")
        self.chrome_path = input('Please specify the chromedriver path : ')
        self.driver = webdriver.Chrome(
            executable_path=self.chrome_path, options=options)
        self.driver.get(URL)
        sleep(2)

    @classmethod
    def get_esgdata(cls):
        '''
        This function calls the _get_websitename() method and imports the\
        corresponding module
        '''
        website_name = cls._get_websitename()
        module = "esgmetrics.esgscraper." + website_name

        __import__(module)

    @staticmethod
    def _get_filename() -> str:
        '''
        This function asks the user for the filepath of the .csv file that
        lists companies name

        Returns:
            str : The input file path
        '''
        companies_filename = input('Enter the filepath of a .csv file with'
                                   + ' company names. To run on example'
                                   + ' dataset(Forbes 2020 2000 companies'
                                   + ' list), enter 0 : ')
        if companies_filename == '0':
            return pkg_resources.resource_stream(__name__, 'data/Forbes.csv')
        else:
            return companies_filename

    @staticmethod
    def _get_headername() -> str:
        '''
        This function asks the user for the name of the header in the .csv
        file that lists companies name

        Returns:
            str : The input header name
        '''
        companies_headername = input(
            'Enter the name of the header that contains the company names.To'
            + ' run on example dataset(Forbes 2020 2000 list), enter 0  : ')
        if companies_headername == '0':
            return 'Name'
        else:
            return companies_headername

    @staticmethod
    def _get_websitename() -> str:
        '''
        This function asks the user for the website name from which data is to
        be extracted

        Returns:
            str : website name
        '''
        website = input(
            'Which website to scrape the data from: MSCI (enter 1),Yahoo'
            + ' Finance (enter 2), CSRHUB (enter 3), S&P Global (enter 4)'
            + ' , SustainAnalytics (enter 5) :')
        if website == '1':
            return 'msci'
        if website == '2':
            return 'yahoo'
        if website == '3':
            return 'csrhub'
        if website == '4':
            return 'snp_global'
        if website == '5':
            return 'sustainanalytics'
        else:
            return print('Enter a number between 1 to 5')

    @staticmethod
    def _get_exportpath() -> str:
        '''
        This function asks the user for the path plus output filename where
        csv file is exported

        Returns:
            str : filepath
        '''
        export_path = input(
            'Enter the path with output csv file name where output csv file is'
            + ' to be exported. : ')
        return export_path

    def wait_element_to_load(self, xpath: str):
        '''
        This function waits until the specified xpath is accessible on the
        website

        Args:
            xpath ('str'): The xpath of the element to be located on the
            webpage

        '''
        delay = 10  # seconds
        ignored_exceptions = (NoSuchElementException,
                              StaleElementReferenceException,)
        try:
            sleep(0.5)
            WebDriverWait(self.driver, delay,
                          ignored_exceptions=ignored_exceptions).until(
                EC.presence_of_element_located((By.XPATH, xpath)))
        except TimeoutException:
            pass

    def accept_cookies(self, xpath: str):
        '''
        This function clicks on 'Accept cookies' button on the website

        Args:
            xpath (str): The xpath of the 'Accept cookies' button
        '''
        cookies_button = self.driver.find_element_by_xpath(xpath)
        WebScraper.wait_element_to_load(self, xpath)
        cookies_button.click()
        sleep(2)

    @staticmethod
    def convert_dict_to_csv(dict_name: str, export_path: str) -> pd.DataFrame:
        '''
        This function converts the dictionary to a pandas dataframe and the
        latter is converted a csv file

        Args:
            dict_name (str): Name of the dictionary
            export_path (str) : Filepath including the outpule filename in
            which csv file is to be exported

        Returns:
            pd.DataFrame: Pandas Dataframe generated from the dictionary
        '''
        df1 = pd.DataFrame.from_dict(dict_name)
        # If the file already exists, append the new data
        if os.path.isfile(export_path + '.csv'):
            df1.to_csv(export_path + '.csv',
                       mode='a', header=False, index=False)
        else:
            df1.to_csv(export_path + '.csv', index=False)
        return df1

    @staticmethod
    def append_empty_values(dictionary: dict) -> dict:
        '''
        This function appends empty values to the dictionary

        Args:
            dict (dict): Dictionary to be appended

        Returns:
            dict: Dictionary appended with empty values
        '''
        for key in dictionary.keys():
            dictionary[key].append(None)
        return dictionary

    def find_element(self, xpath: str = None, class_name: str = None,
                     multiple: bool = False) -> WebElement:
        '''
        Given xpath or class name, this function locates the corresponding web
        element

        Args:
            xpath (str) : xpath of the web element
            class_name (str) : class name of the web element
            multiple (bool) : True if multiple elements to be located; False
            otherwise

        Returns:
            WebElement: element on the website
        '''
        if xpath and multiple:
            WebScraper.wait_element_to_load(self, xpath)
            return self.driver.find_elements_by_xpath(xpath)
        elif xpath and not multiple:
            WebScraper.wait_element_to_load(self, xpath)
            return self.driver.find_element_by_xpath(xpath)
        elif class_name and multiple:
            return self.driver.find_elements_by_class_name(class_name)
        elif class_name and not multiple:
            return self.driver.find_element_by_class_name(class_name)
        return None

    def send_request_to_search_bar(self, header_name, df: pd.DataFrame, i: int,
                                   xpath: str = None, class_name: int =
                                   None) -> WebElement:
        '''
        Given xpath or class name, this function locates the search bar
        and enters the company name

        Args:
            df (dataframe) : input pandas datframe containing Companies name
            xpath (str) : xpath of the search bar
            class_name (str) : class name of the search bar
            multiple (bool) : True if multiple elements to be located;
            False otherwise

        Returns:
            WebElement: webelement of the search bar
        '''
        Company = str(df.loc[i][header_name])
        search_bar = WebScraper.find_element(self, xpath, class_name)
        search_bar.clear()
        search_bar = WebScraper.find_element(self, xpath, class_name)
        search_bar.send_keys(Company)
        sleep(3)
        search_bar = WebScraper.find_element(self, xpath, class_name)
        return search_bar

    def restart_driver(self, cookies_xpath):
        '''
        This function restarts the website

        Args:
            xpath (str): The xpath of the 'Accept cookies' button

        Returns:
            WebScraper instance
        '''
        self.driver.quit()
        sleep(60)
        bot = WebScraper(self.URL)
        bot.accept_cookies(cookies_xpath)
        return bot
